import os
import sys



def getFilePathMap():
    path = os.path.join(os.path.abspath(__file__),'..','..')
    map = {}
    for file in os.listdir(os.path.join(path,'gitignore')):
        if file.endswith('.gitignore'):
            map[file[:file.rindex('.')].lower()] = os.path.join(path,'gitignore',file)
        elif file == 'Global':
            for globalFile in os.listdir(os.path.join(path,'gitignore','Global')):
                if globalFile.endswith('.gitignore'):
                    map[globalFile[:globalFile.rindex('.')]] = os.path.join(path,'gitignore', 'Global', globalFile)
    return map

def getFileContent(path):
    with open(path, 'r') as fr:
        return fr.read()



#import net
import argparse
import os
from pygig import file_tool as ft


os.environ["GIT_PYTHON_REFRESH"] = "quiet"
import git

gitExePath = 'D:\\Git\\Git\\bin\\git.exe'

def listFile(searchStr = None):
    """
    get all .gitignore files if searchStr is None
    get specific .gitignore files including searchStr if searchStr is not None
    """
    # map = net.getFileUrlMap()
    map = ft.getFilePathMap()
    if searchStr is None:
        return '\n'.join(map.keys())
    else:
        return '\n'.join([x for x in map.keys() if x.lower().find(searchStr.lower()) != -1])

def showFile(fileName):
    """show the content of given .gitignore file"""
    # map = net.getFileUrlMap()
    map = ft.getFilePathMap()
    lowerKeyMap = {}
    for key in map:
        lowerKeyMap[key.lower()] = map[key]
    # print(net.getFileContent(map[fileName.strip().lower()]))
    print(ft.getFileContent(lowerKeyMap[fileName.strip().lower()]))

def genFile(fileNames, path):
    """generate .gitignore files in fileNames at path"""
    #map = net.getFileUrlMap()
    map = ft.getFilePathMap()
    lowerKeyMap = {}
    for key in map:
        lowerKeyMap[key.lower()] = map[key]
    content = []
    for fileName in fileNames:
        # content.append(net.getFileContent(map[fileName.strip()]))
        content.append(ft.getFileContent(lowerKeyMap[fileName.strip().lower()]))
    with open(os.path.join(path,'.gitignore'), 'w', newline='') as fw:
        fw.write('\n\n'.join(content))

def updateIgnoreFiles():
    git.refresh(gitExePath)
    repo = git.Repo(os.path.join(os.path.abspath(__file__),'..','..','gitignore'))
    remote = repo.remote()
    remote.pull()

def entrance():
    parser = argparse.ArgumentParser()
    sub_parsers = parser.add_subparsers(dest='command', metavar='command')
    sub_parsers.required = True
    sub_parser = sub_parsers.add_parser('list',
                                        help='list all .gitignore file can be generated for you')
    sub_parser.add_argument("-s", "--search", help="search .gitignore files including these words")
    sub_parser = sub_parsers.add_parser('show',
                                        help='show the content of given .gitignore file')
    sub_parser.add_argument("name", help=".gitignore file name")
    sub_parser = sub_parsers.add_parser('gen',
                                        help='generate .gitignore files')
    sub_parser.add_argument("fileNames", help=".gitignore files' name")
    sub_parser.add_argument("-p", "--path", default='.', help="the directory to generate .gitignore file")
    sub_parser = sub_parsers.add_parser('update',
                                        help='update gitignore directory')
    args = parser.parse_args()
    if args.command == 'list':
        print(listFile(args.search))
    elif args.command == 'show':
        if args.name is None:
            print('please input .gitignore file name')
        else:
            showFile(args.name)
    elif args.command == 'gen':
        fileNames = args.fileNames.split(',')
        genFile(fileNames, args.path)
    elif args.command == 'update':
        updateIgnoreFiles()




import urllib.request
from bs4 import BeautifulSoup

basurl = 'https://github.com'


def getFileUrlMap():
    doc = urllib.request.urlopen(basurl + '/github/gitignore')

    data = doc.read()  # 读取全部
    bsObj = BeautifulSoup(data, "html.parser")
    aList = bsObj.findAll("a", {"class": "js-navigation-open"})

    map = {}
    for a in aList:
        if len(a['class']) > 1 or a.string == '.github' or a.string == 'README.md' or a.string == 'LICENSE':
            continue
        if a.string == 'Global':
            data2 = urllib.request.urlopen(basurl + a['href']).read()
            aList2 = BeautifulSoup(data2, "html.parser").findAll("a", {"class": "js-navigation-open"})
            for a2 in aList2:
                if len(a2['class']) > 1 or a2.string == '..' or a2.string == 'README.md':
                    continue
                s = str(a2.string)
                s = s[:s.rindex('.')].lower()
                map[s] = basurl + a2['href']
        else:
            s = str(a.string)
            s = s[:s.rindex('.')].lower()
            map[s] = basurl + a['href']
    return map

def getFileContent(url):
    doc = urllib.request.urlopen(url)

    data = doc.read()
    bsObj = BeautifulSoup(data, "html.parser")
    table = bsObj.findAll("table", {"class": "js-file-line-container"})
    content = []
    for line in table[0].findAll("td", {"class": "js-file-line"}):
        content.append(line.string)
    return '\n'.join(content)



# coding: utf-8

from setuptools import setup

setup(
    name='pygig',
    version='1.0.7',
    author='lyq',
    author_email='1033020837@qq.com',
    url='https://github.com/1033020837/GitIgnoreGenerator',
    description='A command tool to generate .gitignore file.',
    packages=['pygig','gitignore'],
    install_requires=['argparse','GitPython'],
    include_package_data=True,
    package_data={"gitignore": ["gitignore/*"]},
    entry_points={
        'console_scripts': [
            'pygig=pygig:main'
        ]
    }
)